//
//  ArtworkCircleAnimator.m
//  NowPlayingSaver
//
//  Created by Juri Pakaste on 17.10.09.
//  Copyright (c) 2009, Juri Pakaste. All rights reserved.
//  For licensing details see the file COPYING.
//

#import "ArtworkCircleAnimator.h"

#import "CAAnimation+BlockAdditions.h"
#import "CADebug.h"

#import <QuartzCore/QuartzCore.h>

@implementation ArtworkCircleAnimator

-(id)initWithDirection:(ArtworkCircleAnimatorDirection)direction
{
    if ((self = [super init]))
    {
        mDirection = direction;
    }
    return self;
}


-(void)performArtworkAnimationInLayer:(CALayer*)layer toArtwork:(CGImageRef)image;
{
    layer.masksToBounds = NO;
    
    CGFloat longerLayerSide = MAX(layer.bounds.size.width, layer.bounds.size.height);
    CGFloat rotaterSide = longerLayerSide * 2 + MAX(CGImageGetWidth(image), CGImageGetHeight(image));
    
    CALayer* rotater = [CALayer layer];
    rotater.bounds = CGRectMake(0, 0, rotaterSide, rotaterSide);

    rotater.position = CGPointMake(layer.bounds.size.width/2, rotaterSide/2);
    rotater.anchorPoint = CGPointMake(0.5, 0.5);
    rotater.masksToBounds = NO;
    [layer addSublayer:rotater];
    
    if (layer.contents != nil)
    {
        CALayer* lold = [CALayer layer];
        lold.bounds = layer.bounds;
        lold.position = CGPointMake(rotaterSide/2, lold.frame.size.height/2.0);
        lold.anchorPoint = CGPointMake(0.5, 0.5);
        lold.contents = layer.contents;
        lold.contentsGravity = kCAGravityResizeAspect;
        [rotater addSublayer:lold];
        layer.contents = nil;
    }
    
    CALayer* lnew = [CALayer layer];
    lnew.bounds = layer.bounds;
    lnew.position = CGPointMake((mDirection == kArtworkCircleClockwise) ? (rotaterSide - (lnew.bounds.size.height/2)) : lnew.bounds.size.height/2, rotaterSide/2);
    lnew.anchorPoint = CGPointMake(0.5, 0.5);
    lnew.transform = CATransform3DMakeRotation((mDirection == kArtworkCircleClockwise) ? M_PI/2 : -M_PI/2, 0, 0, 1);
    lnew.contents = (id)image;
    lnew.contentsGravity = kCAGravityResizeAspect;
    [rotater addSublayer:lnew];
    
    CABasicAnimation *a1;
    
    a1 = [CABasicAnimation animationWithKeyPath:@"transform.rotation.z"];
    a1.fromValue = [NSNumber numberWithFloat:0];
    a1.toValue = [NSNumber numberWithFloat:(mDirection == kArtworkCircleClockwise) ? -M_PI/2 : M_PI/2];
    a1.timingFunction = [CAMediaTimingFunction functionWithName:kCAMediaTimingFunctionEaseInEaseOut];
    a1.removedOnCompletion = NO;
    a1.duration = 2;
    a1.fillMode = kCAFillModeForwards;
    
    [a1 setEndBlock:[^(CAAnimation *anim, BOOL finished) {
        layer.contents = (id)image;
        layer.contentsGravity = kCAGravityResizeAspect;
        [rotater removeFromSuperlayer];
    } copy]];
    
    [rotater addAnimation:a1 forKey:@"rotateCircle"];
}

@end
