//
//  ArtworkDoorwayAnimator.m
//  NowPlayingSaver
//
//  Created by Juri Pakaste on 18.10.09.
//  Copyright (c) 2009, Juri Pakaste. All rights reserved.
//  For licensing details see the file COPYING.
//

#import "ArtworkDoorwayAnimator.h"

#import "CAAnimation+BlockAdditions.h"
#import "CADebug.h"

#import <QuartzCore/QuartzCore.h>
#import <math.h>

@implementation ArtworkDoorwayAnimator

-(void)performArtworkAnimationInLayer:(CALayer*)layer toArtwork:(CGImageRef)image;
{
    CGFloat linePos = fmod(arc4random(), layer.bounds.size.height);
    
    CALayer* lineLayer = [CALayer layer];
    [CATransaction begin];
    [CATransaction setDisableActions:YES];

    CGFloat lineWidth = CGImageGetWidth(image);
    if (layer.contents != nil)
    {
        CGImageRef oldImage = (CGImageRef)layer.contents;
        CGFloat oldWidth = CGImageGetWidth(oldImage);
        if (oldWidth > lineWidth)
            lineWidth = oldWidth;
    }
    
    lineLayer.bounds = CGRectMake(0, 0, lineWidth, 3);
    lineLayer.anchorPoint = CGPointMake(0, 0);
    lineLayer.position = CGPointMake(layer.bounds.size.width/2-lineWidth/2, linePos);
    CGColorRef white = (CGColorRef)[NSMakeCollectable(CGColorCreateGenericRGB(1.0, 1.0, 1.0, 1.0)) autorelease];
    lineLayer.backgroundColor = white;
    lineLayer.shadowColor = white;
    lineLayer.shadowOpacity = 0.0;
    lineLayer.shadowRadius = 4.0;
    lineLayer.shadowOffset = CGSizeMake(0, 0);
    lineLayer.masksToBounds = NO;
    [CATransaction commit];
    
    CABasicAnimation* oa = [CABasicAnimation animationWithKeyPath:@"opacity"];
    oa.fromValue = [NSNumber numberWithFloat:0.0];
    oa.toValue = [NSNumber numberWithFloat:1.0];
    oa.duration = 2;
    oa.timingFunction = [CAMediaTimingFunction functionWithName:kCAMediaTimingFunctionEaseIn];
    
    CABasicAnimation* sra1 = [CABasicAnimation animationWithKeyPath:@"shadowOpacity"];
    sra1.fromValue = [NSNumber numberWithFloat:0.0];
    sra1.toValue = [NSNumber numberWithFloat:1.0];
    sra1.duration = 1.5;
    sra1.timingFunction = [CAMediaTimingFunction functionWithName:kCAMediaTimingFunctionEaseIn];
    sra1.beginTime = 1;
    sra1.removedOnCompletion = NO;
    sra1.fillMode = kCAFillModeBoth;
    
    CABasicAnimation* sra2 = [CABasicAnimation animationWithKeyPath:@"shadowOpacity"];
    sra2.fromValue = [NSNumber numberWithFloat:1.0];
    sra2.toValue = [NSNumber numberWithFloat:0.0];
    sra2.duration = 1.5;
    sra2.timingFunction = [CAMediaTimingFunction functionWithName:kCAMediaTimingFunctionEaseOut];
    sra2.beginTime = 2.75;
    sra2.removedOnCompletion = NO;
    sra2.fillMode = kCAFillModeBoth;
    
    CAAnimationGroup* lineGroup = [CAAnimationGroup animation];
    lineGroup.animations = [NSArray arrayWithObjects:oa, sra1, sra2, nil];
    lineGroup.duration = 4.25;
    lineGroup.removedOnCompletion = NO;
    lineGroup.fillMode = kCAFillModeForwards;

    [lineGroup setEndBlock:[^(CAAnimation *anim, BOOL finished) {
        CGFloat halfline = lineLayer.bounds.size.height/2;
        CGSize lsize = layer.bounds.size;
        CGRect rect1, rect2;
        
        CGRectDivide(layer.bounds, &rect1, &rect2, linePos, CGRectMinYEdge);
        rect1.size.height -= halfline;
        rect2.origin.y += halfline;
        rect2.size.height -= halfline;

        NSLog(@"linepos: %f", linePos);
        NSLog(@"rect1: %@", NSStringFromCGRect(rect1));
        NSLog(@"rect2: %@", NSStringFromCGRect(rect2));
        
        CALayer* l1 = [CALayer layer];
        CALayer* l2 = [CALayer layer];
        
        CGColorRef black = (CGColorRef)[NSMakeCollectable(CGColorCreateGenericRGB(0.0, 0.0, 0.0, 1.0)) autorelease];

        l1.bounds = rect1;
        l1.position = CGPointZero;
        l1.anchorPoint = CGPointZero;
        l1.backgroundColor = black;
        
        l2.bounds = CGRectOffset(rect2, -rect2.origin.x, -rect2.origin.y);
        l2.position = rect2.origin;
        l2.anchorPoint = CGPointZero;
        l2.backgroundColor = black;
        
        if (layer.contents != nil)
        {
            CGImageRef c = (CGImageRef)layer.contents;
            
            // images have flipped coordinate system
            CGRect irect1, irect2;
            CGFloat flinepos = lsize.height - linePos;
            
            CGRectDivide(layer.bounds, &irect1, &irect2, flinepos, CGRectMinYEdge);
            irect1.size.height -= halfline;
            irect2.origin.y += halfline;
            irect2.size.height -= halfline;
            
            l1.contents = (id)CGImageCreateWithImageInRect(c, irect2);
            l1.contentsGravity = kCAGravityResizeAspect;
            l2.contents = (id)CGImageCreateWithImageInRect(c, irect1);
            l2.contentsGravity = kCAGravityResizeAspect;
        }
        [layer addSublayer:l1];
        [layer addSublayer:l2];
        
        layer.contents = (id)image;
        layer.contentsGravity = kCAGravityResizeAspect;
        
        CGFloat lineRatio = lsize.height / linePos;
        
        CABasicAnimation *opena1, *opena2, *lineFade;
        
        opena1 = [CABasicAnimation animationWithKeyPath:@"position"];
        opena1.fromValue = [NSValue valueWithPoint:l1.position];
        opena1.toValue = [NSValue valueWithPoint:CGPointMake(0, -l1.bounds.size.height)];
        opena1.duration = 1.5 * lineRatio;
        opena1.timingFunction = [CAMediaTimingFunction functionWithName:kCAMediaTimingFunctionEaseIn];
        opena1.removedOnCompletion = NO;
        opena1.fillMode = kCAFillModeForwards;
        
        [opena1 setEndBlock:[^(CAAnimation *anim, BOOL finished) {
            [l1 removeFromSuperlayer];
        } copy]];
        
        opena2 = [CABasicAnimation animationWithKeyPath:@"position"];
        opena2.fromValue = [NSValue valueWithPoint:l2.position];
        opena2.toValue = [NSValue valueWithPoint:CGPointMake(0, lsize.height)];
        opena2.duration = 1.5 * lineRatio;
        opena2.timingFunction = [CAMediaTimingFunction functionWithName:kCAMediaTimingFunctionEaseIn];
        opena2.removedOnCompletion = NO;
        opena2.fillMode = kCAFillModeForwards;
        
        [opena2 setEndBlock:[^(CAAnimation *anim, BOOL finished) {
            [l2 removeFromSuperlayer];
        } copy]];
        
        lineFade = [CABasicAnimation animationWithKeyPath:@"opacity"];
        lineFade.fromValue = [NSNumber numberWithFloat:1.0];
        lineFade.toValue = [NSNumber numberWithFloat:0.0];
        lineFade.timingFunction = [CAMediaTimingFunction functionWithName:kCAMediaTimingFunctionEaseInEaseOut];
        lineFade.duration = 0.25;
        lineFade.removedOnCompletion = NO;
        lineFade.fillMode = kCAFillModeForwards;
        
        [lineFade setEndBlock:[^(CAAnimation *anim, BOOL finished) {
            [lineLayer removeFromSuperlayer];
        } copy]];
        
        [l1 addAnimation:opena1 forKey:@"open"];
        [l2 addAnimation:opena2 forKey:@"open"];
        [lineLayer addAnimation:lineFade forKey:@"fade"];
    } copy]];
    
    [layer addSublayer:lineLayer];
    [lineLayer addAnimation:lineGroup forKey:@"dividerLineAnimation"];
}

@end
