//
//  ArtworkDrillAnimator.m
//  NowPlayingSaver
//
//  Created by Juri Pakaste on 6.10.09.
//  Copyright (c) 2009, Juri Pakaste. All rights reserved.
//  For licensing details see the file COPYING.
//

#import "ArtworkDrillAnimator.h"

#import "CAAnimation+BlockAdditions.h"

#import <QuartzCore/QuartzCore.h>

@implementation ArtworkDrillAnimator

-(void)performArtworkAnimationInLayer:(CALayer*)layer toArtwork:(CGImageRef)image;
{
    BOOL hasOld = layer.contents != nil;    
    CABasicAnimation *sa1, *sa2;
    CAKeyframeAnimation *ra1, *ra2;
    CAAnimationGroup *a1, *a2;
    
    sa1 = [CABasicAnimation animationWithKeyPath:@"transform.scale"];
    sa1.fromValue = [NSNumber numberWithFloat:1.0];
    sa1.toValue = [NSNumber numberWithFloat:0.0001];
    sa1.duration = 2;
    sa1.timingFunction = [CAMediaTimingFunction functionWithName:kCAMediaTimingFunctionLinear];
    
    sa2 = [CABasicAnimation animationWithKeyPath:@"transform.scale"];
    sa2.fromValue = [NSNumber numberWithFloat:0.0001];
    sa2.toValue = [NSNumber numberWithFloat:1.0];
    sa2.duration = 2;
    sa2.timingFunction = [CAMediaTimingFunction functionWithName:kCAMediaTimingFunctionLinear];
        
    ra1 = [CAKeyframeAnimation animationWithKeyPath:@"transform.rotation.z"];
    ra1.values = [NSArray arrayWithObjects:
                  [NSNumber numberWithFloat:0.0],
                  [NSNumber numberWithFloat:M_PI*2],
                  nil];
    ra1.duration = 1;
    ra1.repeatCount = INT_MAX;
    ra1.timingFunction = [CAMediaTimingFunction functionWithName:kCAMediaTimingFunctionLinear];
    
    ra2 = [CAKeyframeAnimation animationWithKeyPath:@"transform.rotation.z"];
    ra2.values = [NSArray arrayWithObjects:
                  [NSNumber numberWithFloat:M_PI*2],
                  [NSNumber numberWithFloat:0],
                  nil];
    ra2.duration = 1;
    ra2.repeatCount = INT_MAX;
    ra2.timingFunction = [CAMediaTimingFunction functionWithName:kCAMediaTimingFunctionLinear];
    
    a1 = [CAAnimationGroup animation];
    a1.animations = [NSArray arrayWithObjects:sa1, ra1, nil];
    a1.duration = 2;
    a1.removedOnCompletion = NO;
    a1.fillMode = kCAFillModeForwards;
    
    a2 = [CAAnimationGroup animation];
    a2.animations = [NSArray arrayWithObjects:sa2, ra2, nil];
    a2.duration = 2;
    a2.removedOnCompletion = NO;
    a2.fillMode = kCAFillModeForwards;
    
    [a2 setStartBlock:[^(CAAnimation *anim) {
        layer.contents = (id)image;
        layer.contentsGravity = kCAGravityResizeAspect;
    } copy]];
    
    [a2 setEndBlock:[^(CAAnimation *anim, BOOL finished) {
        [layer removeAnimationForKey:@"grow"];
    } copy]];
    
    if (hasOld)
    {
        [a1 setEndBlock:[^(CAAnimation* anim, BOOL finished) {
            [layer removeAnimationForKey:@"shrink"];
            [layer addAnimation:a2 forKey:@"grow"];
        } copy]];
        [layer addAnimation:a1 forKey:@"shrink"];
        
    }
    else
    {
        [layer addAnimation:a2 forKey:@"grow"];
    }
}

@end
