//
//  ArtworkFadeOutInAnimator.m
//  NowPlayingSaver
//
//  Created by Juri Pakaste on 19.9.09.
//  Copyright (c) 2009, Juri Pakaste. All rights reserved.
//  For licensing details see the file COPYING.
//

#import "ArtworkFadeOutInAnimator.h"

#import "CAAnimation+BlockAdditions.h"
#import "CADebug.h"

#import <QuartzCore/QuartzCore.h>


@implementation ArtworkFadeOutInAnimator

-(void)performArtworkAnimationInLayer:(CALayer*)layer toArtwork:(CGImageRef)image;
{
    BOOL hasOld = layer.contents != nil;
    CABasicAnimation *a1, *a2;

    a1 = [CABasicAnimation animationWithKeyPath:@"opacity"];
    a1.fromValue = [NSNumber numberWithFloat:1.0];
    a1.toValue = [NSNumber numberWithFloat:0.0];
    a1.duration = 2;
    a1.timingFunction = [CAMediaTimingFunction functionWithName:kCAMediaTimingFunctionEaseIn];
    a1.removedOnCompletion = NO;
    a1.fillMode = kCAFillModeBoth;

    a2 = [CABasicAnimation animationWithKeyPath:@"opacity"];
    a2.fromValue = [NSNumber numberWithFloat:0.0];
    a2.toValue = [NSNumber numberWithFloat:1.0];
    a2.duration = 2;
    a2.timingFunction = [CAMediaTimingFunction functionWithName:kCAMediaTimingFunctionEaseOut];
    a2.removedOnCompletion = NO;
    a2.fillMode = kCAFillModeForwards;

    [a2 setStartBlock:[^(CAAnimation *anim) {
        layer.contents = (id)image;
        layer.contentsGravity = kCAGravityResizeAspect;
    } copy]];
    
    [a2 setEndBlock:[^(CAAnimation *anim, BOOL finished) {
        [layer removeAnimationForKey:@"fadein"];
    } copy]];
    
    if (hasOld)
    {    
        [a1 setEndBlock:[^(CAAnimation *anim, BOOL finished) {
            [layer removeAnimationForKey:@"fadeout"];
            [layer addAnimation:a2 forKey:@"fadein"];
        } copy]];
        
        [layer addAnimation:a1 forKey:@"fadeout"];        
    }
    else
    {
        [layer addAnimation:a2 forKey:@"fadein"];
    }

}
 
@end
