//
//  ArtworkFlipAnimator.m
//  NowPlayingSaver
//
//  Created by Juri Pakaste on 7.10.09.
//  Copyright (c) 2009, Juri Pakaste. All rights reserved.
//  For licensing details see the file COPYING.
//

#import "ArtworkFlipAnimator.h"

#import "CAAnimation+BlockAdditions.h"
#import "CADebug.h"

#import <QuartzCore/QuartzCore.h>

static const NSInteger kPerspectiveDepth = 1000;

@implementation ArtworkFlipAnimator

-(id)initWithAxis:(ArtworkFlipAnimatorAxis)axis
{
    if ((self = [super init]))
    {
        mAxis = axis;
    }
    return self;
}


-(void)performArtworkAnimationInLayer:(CALayer*)layer toArtwork:(CGImageRef)image;
{
    BOOL hasOld = layer.contents != nil;
    
    CATransform3D t = layer.transform;
    t.m34 = 1.0 / -kPerspectiveDepth;
    layer.transform = t;
    
    CABasicAnimation *a1, *a2;
    
    NSString* keyPath = nil;
    switch (mAxis) {
        case kArtworkFlipAxisX:
            keyPath = @"transform.rotation.x";
            break;
        case kArtworkFlipAxisY:
            keyPath = @"transform.rotation.y";
            break;
    }
    
    a1 = [CABasicAnimation animationWithKeyPath:keyPath];
    a1.fromValue = [NSNumber numberWithFloat:0.0];
    a1.toValue = [NSNumber numberWithFloat:M_PI/2.0];
    a1.duration = hasOld ? 2 : 0;
    a1.timingFunction = [CAMediaTimingFunction functionWithName:kCAMediaTimingFunctionEaseIn];
    a1.removedOnCompletion = NO;
    a1.fillMode = kCAFillModeForwards;
    
    a2 = [CABasicAnimation animationWithKeyPath:keyPath];
    a2.fromValue = [NSNumber numberWithFloat:M_PI/2.0];
    a2.toValue = [NSNumber numberWithFloat:0];
    a2.duration = 2;
    a2.timingFunction = [CAMediaTimingFunction functionWithName:kCAMediaTimingFunctionEaseOut];
    a2.removedOnCompletion = NO;
    a2.fillMode = kCAFillModeForwards;
    
    [a2 setStartBlock:[^(CAAnimation *anim) {
        [CATransaction begin];
        [CATransaction setDisableActions:YES];
        layer.contents = (id)image;
        layer.contentsGravity = kCAGravityResizeAspect;

        CATransform3D t = layer.transform;
        t.m34 = 1.0 / kPerspectiveDepth;
        layer.transform = t;
        [CATransaction commit];        
    } copy]];
    
    [a2 setEndBlock:[^(CAAnimation *anim, BOOL finished) {
        [layer removeAnimationForKey:@"rotatein"];

        CATransform3D t = layer.transform;
        t.m34 = 0.0;
        layer.transform = t;
    } copy]];
    
    [a1 setEndBlock:[^(CAAnimation* anim, BOOL finished) {
        [layer removeAnimationForKey:@"rotateout"];
        [layer addAnimation:a2 forKey:@"rotatein"];
    } copy]];

    [layer addAnimation:a1 forKey:@"rotateout"];
}

@end
