//
//  ArtworkFromSideAnimator.m
//  NowPlayingSaver
//
//  Created by Juri Pakaste on 10.10.09.
//  Copyright (c) 2009, Juri Pakaste. All rights reserved.
//  For licensing details see the file COPYING.
//

#import "ArtworkFromSideAnimator.h"

#import "CAAnimation+BlockAdditions.h"
#import "CADebug.h"

#import <QuartzCore/QuartzCore.h>

@implementation ArtworkFromSideAnimator

-(id)initWithDirection:(ArtworkFromSideDirection)direction
{
    if ((self = [super init]))
    {
        mDirection = direction;
    }
    return self;
}


-(void)performArtworkAnimationInLayer:(CALayer*)layer toArtwork:(CGImageRef)image;
{
    BOOL hasOld = layer.contents != nil;
    
    CALayer* lnew = [CALayer layer];
    lnew.bounds = layer.bounds;
    lnew.position = CGPointMake(((mDirection == kArtworkFromSideRight) ? layer.bounds.size.width : -layer.bounds.size.width), 0);
    lnew.anchorPoint = CGPointMake(0, 0);
    lnew.contents = (id)image;
    lnew.contentsGravity = kCAGravityResizeAspect;

    CABasicAnimation *a1, *a2;
    
    a2 = [CABasicAnimation animationWithKeyPath:@"position"];
    a2.fromValue = [NSValue valueWithPoint:lnew.position];
    a2.toValue = [NSValue valueWithPoint:CGPointZero];
    a2.timingFunction = [CAMediaTimingFunction functionWithName:kCAMediaTimingFunctionEaseInEaseOut];
    a2.duration = 2;
    a2.removedOnCompletion = NO;
    a2.fillMode = kCAFillModeForwards;
    
    [a2 setEndBlock:[^(CAAnimation *anim, BOOL finished) {
        layer.contents = lnew.contents;
        layer.contentsGravity = kCAGravityResizeAspect;
        layer.anchorPoint = CGPointMake(0.5, 0.5);
        [lnew removeFromSuperlayer];
    } copy]];
    
    if (hasOld)
    {
        CALayer* lold = [CALayer layer];
        lold.bounds = layer.bounds;
        lold.position = CGPointZero;
        lold.anchorPoint = CGPointZero;
        lold.contents = layer.contents;
        lold.contentsGravity = kCAGravityResizeAspect;
        
        a1 = [CABasicAnimation animationWithKeyPath:@"position"];
        a1.fromValue = [NSValue valueWithPoint:lold.position];
        a1.toValue = [NSValue valueWithPoint:CGPointMake(((mDirection == kArtworkFromSideRight) ? -layer.bounds.size.width : layer.bounds.size.width), lold.position.y)];
        a1.timingFunction = [CAMediaTimingFunction functionWithName:kCAMediaTimingFunctionEaseInEaseOut];
        a1.removedOnCompletion = NO;
        a1.duration = 2;
        a1.fillMode = kCAFillModeForwards;

        [a1 setEndBlock:[^(CAAnimation *anim, BOOL finished) {
            [lold removeFromSuperlayer];
        } copy]];
        
        [layer addSublayer:lold];
        layer.contents = nil;

        [lold addAnimation:a1 forKey:@"moveOld"];
    }
    [layer addSublayer:lnew];    
    [lnew addAnimation:a2 forKey:@"moveNew"];
}

@end
