//
//  ArtworkScaleDownUpAnimator.m
//  NowPlayingSaver
//
//  Created by Juri Pakaste on 19.9.09.
//  Copyright (c) 2009, Juri Pakaste. All rights reserved.
//  For licensing details see the file COPYING.
//

#import "ArtworkScaleDownUpAnimator.h"

#import "CAAnimation+BlockAdditions.h"
#import "CADebug.h"

#import <QuartzCore/QuartzCore.h>

@implementation ArtworkScaleDownUpAnimator

-(void)performArtworkAnimationInLayer:(CALayer*)layer toArtwork:(CGImageRef)image;
{
    BOOL hasOld = layer.contents != nil;
    CATransform3D zeroScale = CATransform3DMakeScale(0.0001, 0.0001, 0.0001);    
    CABasicAnimation *a1, *a2;
    
    a1 = [CABasicAnimation animationWithKeyPath:@"transform"];
    a1.fromValue = [NSValue valueWithCATransform3D:CATransform3DIdentity];
    a1.toValue = [NSValue valueWithCATransform3D:zeroScale];    
    a1.duration = 2;
    a1.timingFunction = [CAMediaTimingFunction functionWithName:kCAMediaTimingFunctionEaseIn];
    a1.removedOnCompletion = NO;
    a1.fillMode = kCAFillModeForwards;

    a2 = [CABasicAnimation animationWithKeyPath:@"transform"];
    a2.fromValue = [NSValue valueWithCATransform3D:zeroScale];
    a2.toValue = [NSValue valueWithCATransform3D:CATransform3DIdentity];
    a2.duration = 2;
    a2.timingFunction = [CAMediaTimingFunction functionWithName:kCAMediaTimingFunctionEaseOut];
    a2.removedOnCompletion = NO;
    a2.fillMode = kCAFillModeForwards;    

    [a2 setStartBlock:[^(CAAnimation *anim) {
        layer.contents = (id)image;
        layer.contentsGravity = kCAGravityResizeAspect;
    } copy]];
    
    [a2 setEndBlock:[^(CAAnimation *anim, BOOL finished) {
        [layer removeAnimationForKey:@"grow"];
    } copy]];
    
    if (hasOld)
    {
        [a1 setEndBlock:[^(CAAnimation* anim, BOOL finished) {
            [layer removeAnimationForKey:@"shrink"];
            [layer addAnimation:a2 forKey:@"grow"];
        } copy]];
        [layer addAnimation:a1 forKey:@"shrink"];
        
    }
    else
    {
        [layer addAnimation:a2 forKey:@"grow"];
    }
}

@end
