//
//  CAAnimation+BlockAdditions.m
//  NowPlayingSaver
//
//  Created by Juri Pakaste on 23.9.09.
//  Copyright (c) 2009, Juri Pakaste. All rights reserved.
//  For licensing details see the file COPYING.
//

#import "CAAnimation+BlockAdditions.h"


@interface CABlockDelegate : NSObject {
@private
    void (^mStartBlock)(CAAnimation* anim);
    void (^mEndBlock)(CAAnimation* anim, BOOL flag);
}

-(void)setStartBlock:(void(^)(CAAnimation* anim))block;
-(void)setEndBlock:(void(^)(CAAnimation* anim, BOOL finished))block;

-(void)animationDidStart:(CAAnimation*)theAnimation;
-(void)animationDidStop:(CAAnimation*)theAnimation finished:(BOOL)flag;

@end


@implementation CAAnimation (BlockAdditions)

-(void)setStartBlock:(void(^)(CAAnimation* anim))block
{
    if ((self.delegate == nil) || (! [self.delegate isKindOfClass:[CABlockDelegate class]]))
    {
        self.delegate = [[CABlockDelegate alloc] init];
    }
    
    [self.delegate setStartBlock:block];
}

-(void)setEndBlock:(void(^)(CAAnimation* anim, BOOL finished))block
{
    if ((self.delegate == nil) || (! [self.delegate isKindOfClass:[CABlockDelegate class]]))
    {
        self.delegate = [[CABlockDelegate alloc] init];
    }
    
    [self.delegate setEndBlock:block];
}

@end


@implementation CABlockDelegate

-(void)animationDidStart:(CAAnimation*)theAnimation
{
    if (mStartBlock != nil)
        mStartBlock(theAnimation);
}

-(void)animationDidStop:(CAAnimation*)theAnimation finished:(BOOL)flag
{
    if (mEndBlock != nil)
        mEndBlock(theAnimation, flag);
}

-(void)setStartBlock:(void (^)(CAAnimation *))block
{
    mStartBlock = block;
}

-(void)setEndBlock:(void (^)(CAAnimation *, BOOL))block
{
    mEndBlock = block;
}

@end

